==> openmsx_0.8.0.orig.tar.gz: openmsx-0.8.0/src/utils/win32-dirent.cc <==
// $Id: win32-dirent.cc 9571 2009-04-13 19:37:05Z mthuurne $

/* Copyright (C) 2001, 2006 Free Software Foundation, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

// NB: Taken from http://www.koders.com/c/fidB38A2E97F60B8A0C903631F8C60078DE8C6C8433.aspx
// Also modified to use Unicode calls explicitly
// Slightly reformatted/simplified to fit openMSX coding style.

#ifdef _WIN32

#include "win32-dirent.hh"
#include "utf8_checked.hh"
#include "MSXException.hh"
#include "StringOp.hh"
#include <windows.h>
#include <cstring>
#include <cstdlib>
#include <exception>

namespace openmsx {

DIR* opendir(const char* name)
{
	if (!name || !*name) return NULL;

	std::wstring nameW = utf8::utf8to16(name);
	if (!StringOp::endsWith(name, "/") &&
	    !StringOp::endsWith(name, "\\")) {
		nameW += L"\\*";
	} else {
		nameW += L"*";
	}

	HANDLE hnd;
	WIN32_FIND_DATAW find;
	if ((hnd = FindFirstFileW(nameW.c_str(), &find)) == INVALID_HANDLE_VALUE) {
		return NULL;
	}

	DIR* dir = new DIR;
	dir->mask = nameW;
	dir->fd = reinterpret_cast<INT_PTR>(hnd);
	dir->data = new WIN32_FIND_DATAW;
	dir->filepos = 0;
	memcpy(dir->data, &find, sizeof(WIN32_FIND_DATAW));
	return dir;
}

dirent* readdir(DIR* dir)
{
	static dirent entry;
	entry.d_ino = 0;
	entry.d_type = 0;

	WIN32_FIND_DATAW* find = static_cast<WIN32_FIND_DATAW*>(dir->data);
	if (dir->filepos) {
		if (!FindNextFileW(reinterpret_cast<HANDLE>(dir->fd), find)) {
			return NULL;
		}
	}

	std::string d_name = utf8::utf16to8(find->cFileName);
	strncpy(entry.d_name, d_name.c_str(),
	        sizeof(entry.d_name) / sizeof(entry.d_name[0]));

	entry.d_off = dir->filepos;
	entry.d_reclen = static_cast<unsigned short>(strlen(entry.d_name));
	dir->filepos++;
	return &entry;
}

int closedir(DIR* dir)
{
	HANDLE hnd = reinterpret_cast<HANDLE>(dir->fd);
	delete static_cast<WIN32_FIND_DATAW*>(dir->data);
	delete dir;
	return FindClose(hnd) ? 0 : -1;
}

void rewinddir(DIR* dir)
{
	HANDLE hnd = reinterpret_cast<HANDLE>(dir->fd);
	WIN32_FIND_DATAW* find = static_cast<WIN32_FIND_DATAW*>(dir->data);

	FindClose(hnd);
	hnd = FindFirstFileW(dir->mask.c_str(), find);
	dir->fd = reinterpret_cast<INT_PTR>(hnd);
	dir->filepos = 0;
}

void seekdir(DIR* dir, off_t offset)
{
	rewinddir(dir);
	for (off_t n = 0; n < offset; ++n) {
		if (FindNextFileW(reinterpret_cast<HANDLE>(dir->fd),
					  static_cast<WIN32_FIND_DATAW*>(dir->data))) {
			dir->filepos++;
		}
	}
}

off_t telldir(DIR* dir)
{
	return dir->filepos;
}

// For correctness on 64-bit Windows, this function would need to maintain an
// internal map of ints to HANDLEs, since HANDLEs are sizeof(void*). It's not
// used at the moment in the openMSX sources, so let's not bother for now.
/*int dirfd(DIR* dir)
{
	return dir->fd;
}*/

} // namespace openmsx

#endif

==> openmsx_0.8.1.orig.tar.gz: openmsx-0.8.1/src/utils/win32-dirent.cc <==
// $Id: win32-dirent.cc 11609 2010-07-22 21:46:22Z m9710797 $

/* Copyright (C) 2001, 2006 Free Software Foundation, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

// NB: Taken from http://www.koders.com/c/fidB38A2E97F60B8A0C903631F8C60078DE8C6C8433.aspx
// Also modified to use Unicode calls explicitly
// Slightly reformatted/simplified to fit openMSX coding style.

#ifdef _WIN32

#include "win32-dirent.hh"
#include "utf8_checked.hh"
#include "MSXException.hh"
#include "StringOp.hh"
#include <windows.h>
#include <cstring>
#include <cstdlib>
#include <exception>

namespace openmsx {

DIR* opendir(const char* name)
{
	if (!name || !*name) return NULL;

	std::wstring nameW = utf8::utf8to16(name);
	if (!StringOp::endsWith(name, '/') &&
	    !StringOp::endsWith(name, "\\")) {
		nameW += L"\\*";
	} else {
		nameW += L"*";
	}

	HANDLE hnd;
	WIN32_FIND_DATAW find;
	if ((hnd = FindFirstFileW(nameW.c_str(), &find)) == INVALID_HANDLE_VALUE) {
		return NULL;
	}

	DIR* dir = new DIR;
	dir->mask = nameW;
	dir->fd = reinterpret_cast<INT_PTR>(hnd);
	dir->data = new WIN32_FIND_DATAW;
	dir->filepos = 0;
	memcpy(dir->data, &find, sizeof(WIN32_FIND_DATAW));
	return dir;
}

dirent* readdir(DIR* dir)
{
	static dirent entry;
	entry.d_ino = 0;
	entry.d_type = 0;

	WIN32_FIND_DATAW* find = static_cast<WIN32_FIND_DATAW*>(dir->data);
	if (dir->filepos) {
		if (!FindNextFileW(reinterpret_cast<HANDLE>(dir->fd), find)) {
			return NULL;
		}
	}

	std::string d_name = utf8::utf16to8(find->cFileName);
	strncpy(entry.d_name, d_name.c_str(),
	        sizeof(entry.d_name) / sizeof(entry.d_name[0]));

	entry.d_off = dir->filepos;
	entry.d_reclen = static_cast<unsigned short>(strlen(entry.d_name));
	dir->filepos++;
	return &entry;
}

int closedir(DIR* dir)
{
	HANDLE hnd = reinterpret_cast<HANDLE>(dir->fd);
	delete static_cast<WIN32_FIND_DATAW*>(dir->data);
	delete dir;
	return FindClose(hnd) ? 0 : -1;
}

void rewinddir(DIR* dir)
{
	HANDLE hnd = reinterpret_cast<HANDLE>(dir->fd);
	WIN32_FIND_DATAW* find = static_cast<WIN32_FIND_DATAW*>(dir->data);

	FindClose(hnd);
	hnd = FindFirstFileW(dir->mask.c_str(), find);
	dir->fd = reinterpret_cast<INT_PTR>(hnd);
	dir->filepos = 0;
}

void seekdir(DIR* dir, off_t offset)
{
	rewinddir(dir);
	for (off_t n = 0; n < offset; ++n) {
		if (FindNextFileW(reinterpret_cast<HANDLE>(dir->fd),
					  static_cast<WIN32_FIND_DATAW*>(dir->data))) {
			dir->filepos++;
		}
	}
}

off_t telldir(DIR* dir)
{
	return dir->filepos;
}

// For correctness on 64-bit Windows, this function would need to maintain an
// internal map of ints to HANDLEs, since HANDLEs are sizeof(void*). It's not
// used at the moment in the openMSX sources, so let's not bother for now.
/*int dirfd(DIR* dir)
{
	return dir->fd;
}*/

} // namespace openmsx

#endif
